@@ -19,11 +19,13 @@ easier to adjust to future upstream changes in the analogous numpy signatures.
 """
 
 from numpy import ndarray
-from pandas.util._validators import (validate_args, validate_kwargs,
-                                     validate_args_and_kwargs)
-from pandas.errors import UnsupportedFunctionCall
-from pandas.core.dtypes.common import is_integer, is_bool
+
 from pandas.compat import OrderedDict
+from pandas.errors import UnsupportedFunctionCall
+from pandas.util._validators import (
+    validate_args, validate_args_and_kwargs, validate_kwargs)
+
+from pandas.core.dtypes.common import is_bool, is_integer
 
 
 class CompatValidator(object):
@@ -187,15 +189,16 @@ def validate_cum_func_with_skipna(skipna, args, kwargs, name):
 ALLANY_DEFAULTS = OrderedDict()
 ALLANY_DEFAULTS['dtype'] = None
 ALLANY_DEFAULTS['out'] = None
+ALLANY_DEFAULTS['keepdims'] = False
 validate_all = CompatValidator(ALLANY_DEFAULTS, fname='all',
                                method='both', max_fname_arg_count=1)
 validate_any = CompatValidator(ALLANY_DEFAULTS, fname='any',
                                method='both', max_fname_arg_count=1)
 
-LOGICAL_FUNC_DEFAULTS = dict(out=None)
+LOGICAL_FUNC_DEFAULTS = dict(out=None, keepdims=False)
 validate_logical_func = CompatValidator(LOGICAL_FUNC_DEFAULTS, method='kwargs')
 
-MINMAX_DEFAULTS = dict(out=None)
+MINMAX_DEFAULTS = dict(out=None, keepdims=False)
 validate_min = CompatValidator(MINMAX_DEFAULTS, fname='min',
                                method='both', max_fname_arg_count=1)
 validate_max = CompatValidator(MINMAX_DEFAULTS, fname='max',
@@ -223,16 +226,32 @@ validate_sort = CompatValidator(SORT_DEFAULTS, fname='sort',
 STAT_FUNC_DEFAULTS = OrderedDict()
 STAT_FUNC_DEFAULTS['dtype'] = None
 STAT_FUNC_DEFAULTS['out'] = None
+
+PROD_DEFAULTS = SUM_DEFAULTS = STAT_FUNC_DEFAULTS.copy()
+SUM_DEFAULTS['keepdims'] = False
+SUM_DEFAULTS['initial'] = None
+
+MEDIAN_DEFAULTS = STAT_FUNC_DEFAULTS.copy()
+MEDIAN_DEFAULTS['overwrite_input'] = False
+MEDIAN_DEFAULTS['keepdims'] = False
+
+STAT_FUNC_DEFAULTS['keepdims'] = False
+
 validate_stat_func = CompatValidator(STAT_FUNC_DEFAULTS,
                                      method='kwargs')
-validate_sum = CompatValidator(STAT_FUNC_DEFAULTS, fname='sort',
+validate_sum = CompatValidator(SUM_DEFAULTS, fname='sum',
                                method='both', max_fname_arg_count=1)
+validate_prod = CompatValidator(PROD_DEFAULTS, fname="prod",
+                                method="both", max_fname_arg_count=1)
 validate_mean = CompatValidator(STAT_FUNC_DEFAULTS, fname='mean',
                                 method='both', max_fname_arg_count=1)
+validate_median = CompatValidator(MEDIAN_DEFAULTS, fname='median',
+                                  method='both', max_fname_arg_count=1)
 
 STAT_DDOF_FUNC_DEFAULTS = OrderedDict()
 STAT_DDOF_FUNC_DEFAULTS['dtype'] = None
 STAT_DDOF_FUNC_DEFAULTS['out'] = None
+STAT_DDOF_FUNC_DEFAULTS['keepdims'] = False
 validate_stat_ddof_func = CompatValidator(STAT_DDOF_FUNC_DEFAULTS,
                                           method='kwargs')
 
@@ -360,3 +379,24 @@ def validate_resampler_func(method, args, kwargs):
                 "{func}() instead".format(func=method)))
         else:
             raise TypeError("too many arguments passed in")
+
+
+def validate_minmax_axis(axis):
+    """
+    Ensure that the axis argument passed to min, max, argmin, or argmax is
+    zero or None, as otherwise it will be incorrectly ignored.
+
+    Parameters
+    ----------
+    axis : int or None
+
+    Raises
+    ------
+    ValueError
+    """
+    ndim = 1  # hard-coded for Index
+    if axis is None:
+        return
+    if axis >= ndim or (axis < 0 and ndim + axis < 0):
+        raise ValueError("`axis` must be fewer than the number of "
+                         "dimensions ({ndim})".format(ndim=ndim))
