@@ -2,26 +2,25 @@
 """
 
 import ast
+from functools import partial
 import tokenize
 
-from functools import partial
 import numpy as np
 
+from pandas.compat import StringIO, lmap, reduce, string_types, zip
+
 import pandas as pd
 from pandas import compat
-from pandas.compat import StringIO, lmap, zip, reduce, string_types
-from pandas.core.base import StringMixin
 from pandas.core import common as com
-import pandas.io.formats.printing as printing
-from pandas.core.reshape.util import compose
+from pandas.core.base import StringMixin
 from pandas.core.computation.ops import (
-    _cmp_ops_syms, _bool_ops_syms,
-    _arith_ops_syms, _unary_ops_syms, is_term)
-from pandas.core.computation.ops import _reductions, _mathops, _LOCAL_TAG
-from pandas.core.computation.ops import Op, BinOp, UnaryOp, Term, Constant, Div
-from pandas.core.computation.ops import UndefinedVariableError, FuncNode
+    _LOCAL_TAG, BinOp, Constant, Div, FuncNode, Op, Term, UnaryOp,
+    UndefinedVariableError, _arith_ops_syms, _bool_ops_syms, _cmp_ops_syms,
+    _mathops, _reductions, _unary_ops_syms, is_term)
 from pandas.core.computation.scope import Scope
 
+import pandas.io.formats.printing as printing
+
 
 def tokenize_string(source):
     """Tokenize a Python source code string.
@@ -103,8 +102,19 @@ def _replace_locals(tok):
     return toknum, tokval
 
 
-def _preparse(source, f=compose(_replace_locals, _replace_booleans,
-                                _rewrite_assign)):
+def _compose2(f, g):
+    """Compose 2 callables"""
+    return lambda *args, **kwargs: f(g(*args, **kwargs))
+
+
+def _compose(*funcs):
+    """Compose 2 or more callables"""
+    assert len(funcs) > 1, 'At least 2 callables must be passed to compose'
+    return reduce(_compose2, funcs)
+
+
+def _preparse(source, f=_compose(_replace_locals, _replace_booleans,
+                                 _rewrite_assign)):
     """Compose a collection of tokenization functions
 
     Parameters
@@ -307,7 +317,14 @@ class BaseExprVisitor(ast.NodeVisitor):
     def visit(self, node, **kwargs):
         if isinstance(node, string_types):
             clean = self.preparser(node)
-            node = ast.fix_missing_locations(ast.parse(clean))
+            try:
+                node = ast.fix_missing_locations(ast.parse(clean))
+            except SyntaxError as e:
+                from keyword import iskeyword
+                if any(iskeyword(x) for x in clean.split()):
+                    e.msg = ("Python keyword not valid identifier"
+                             " in numexpr query")
+                raise e
 
         method = 'visit_' + node.__class__.__name__
         visitor = getattr(self, method)
@@ -361,11 +378,11 @@ class BaseExprVisitor(ast.NodeVisitor):
 
     def _maybe_downcast_constants(self, left, right):
         f32 = np.dtype(np.float32)
-        if left.isscalar and not right.isscalar and right.return_type == f32:
+        if left.is_scalar and not right.is_scalar and right.return_type == f32:
             # right is a float32 array, left is a scalar
             name = self.env.add_tmp(np.float32(left.value))
             left = self.term_type(name, self.env)
-        if right.isscalar and not left.isscalar and left.return_type == f32:
+        if right.is_scalar and not left.is_scalar and left.return_type == f32:
             # left is a float32 array, right is a scalar
             name = self.env.add_tmp(np.float32(right.value))
             right = self.term_type(name, self.env)
@@ -694,8 +711,8 @@ _numexpr_supported_calls = frozenset(_reductions + _mathops)
 class PandasExprVisitor(BaseExprVisitor):
 
     def __init__(self, env, engine, parser,
-                 preparser=partial(_preparse, f=compose(_replace_locals,
-                                                        _replace_booleans))):
+                 preparser=partial(_preparse, f=_compose(_replace_locals,
+                                                         _replace_booleans))):
         super(PandasExprVisitor, self).__init__(env, engine, parser, preparser)
 
 
