@@ -1,42 +1,39 @@
 # coding=utf-8
 # pylint: disable-msg=E1101,W0612
 
-import pytest
-
-from datetime import datetime, timedelta
 from collections import OrderedDict
+from datetime import datetime, timedelta
 
-from numpy import nan
 import numpy as np
+from numpy import nan
 import numpy.ma as ma
-import pandas as pd
-
-from pandas.api.types import CategoricalDtype
-from pandas.core.dtypes.common import (
-    is_categorical_dtype,
-    is_datetime64tz_dtype)
-from pandas import (Index, Series, isna, date_range, Timestamp,
-                    NaT, period_range, timedelta_range, MultiIndex,
-                    IntervalIndex, Categorical, DataFrame)
+import pytest
 
 from pandas._libs import lib
 from pandas._libs.tslib import iNaT
+from pandas.compat import PY36, long, lrange, range, zip
 
-from pandas.compat import lrange, range, zip, long, PY36
-from pandas.util.testing import assert_series_equal
+from pandas.core.dtypes.common import (
+    is_categorical_dtype, is_datetime64tz_dtype)
+
+import pandas as pd
+from pandas import (
+    Categorical, DataFrame, Index, IntervalIndex, MultiIndex, NaT, Series,
+    Timestamp, date_range, isna, period_range, timedelta_range)
+from pandas.api.types import CategoricalDtype
+from pandas.core.arrays import period_array
 import pandas.util.testing as tm
-
-from .common import TestData
+from pandas.util.testing import assert_series_equal
 
 
-class TestSeriesConstructors(TestData):
+class TestSeriesConstructors():
 
     def test_invalid_dtype(self):
         # GH15520
         msg = 'not understood'
         invalid_list = [pd.Timestamp, 'pd.Timestamp', list]
         for dtype in invalid_list:
-            with tm.assert_raises_regex(TypeError, msg):
+            with pytest.raises(TypeError, match=msg):
                 Series([], name='time', dtype=dtype)
 
     def test_scalar_conversion(self):
@@ -50,26 +47,28 @@ class TestSeriesConstructors(TestData):
         assert int(Series([1.])) == 1
         assert long(Series([1.])) == 1
 
-    def test_constructor(self):
-        assert self.ts.index.is_all_dates
+    def test_constructor(self, datetime_series, empty_series):
+        assert datetime_series.index.is_all_dates
 
         # Pass in Series
-        derived = Series(self.ts)
+        derived = Series(datetime_series)
         assert derived.index.is_all_dates
 
-        assert tm.equalContents(derived.index, self.ts.index)
+        assert tm.equalContents(derived.index, datetime_series.index)
         # Ensure new index is not created
-        assert id(self.ts.index) == id(derived.index)
+        assert id(datetime_series.index) == id(derived.index)
 
         # Mixed type Series
         mixed = Series(['hello', np.NaN], index=[0, 1])
         assert mixed.dtype == np.object_
         assert mixed[1] is np.NaN
 
-        assert not self.empty.index.is_all_dates
+        assert not empty_series.index.is_all_dates
         assert not Series({}).index.is_all_dates
-        pytest.raises(Exception, Series, np.random.randn(3, 3),
-                      index=np.arange(3))
+
+        # exception raised is of type Exception
+        with pytest.raises(Exception, match="Data must be 1-dimensional"):
+            Series(np.random.randn(3, 3), index=np.arange(3))
 
         mixed.name = 'Series'
         rs = Series(mixed).name
@@ -78,7 +77,9 @@ class TestSeriesConstructors(TestData):
 
         # raise on MultiIndex GH4187
         m = MultiIndex.from_arrays([[1, 2], [3, 4]])
-        pytest.raises(NotImplementedError, Series, m)
+        msg = "initializing a Series from a MultiIndex is not supported"
+        with pytest.raises(NotImplementedError, match=msg):
+            Series(m)
 
     @pytest.mark.parametrize('input_class', [list, dict, OrderedDict])
     def test_constructor_empty(self, input_class):
@@ -137,6 +138,17 @@ class TestSeriesConstructors(TestData):
         result = pd.Series(index=['b', 'a', 'c'])
         assert result.index.tolist() == ['b', 'a', 'c']
 
+    def test_constructor_no_data_string_type(self):
+        # GH 22477
+        result = pd.Series(index=[1], dtype=str)
+        assert np.isnan(result.iloc[0])
+
+    @pytest.mark.parametrize('item', ['entry', 'ѐ', 13])
+    def test_constructor_string_element_string_type(self, item):
+        # GH 22477
+        result = pd.Series(item, index=[1], dtype=str)
+        assert result.iloc[0] == str(item)
+
     def test_constructor_dtype_str_na_values(self, string_dtype):
         # https://github.com/pandas-dev/pandas/issues/21083
         ser = Series(['x', None], dtype=string_dtype)
@@ -156,12 +168,29 @@ class TestSeriesConstructors(TestData):
 
         assert_series_equal(s2, s1.sort_index())
 
-    def test_constructor_iterator(self):
+    def test_constructor_iterable(self):
+        # GH 21987
+        class Iter():
+            def __iter__(self):
+                for i in range(10):
+                    yield i
 
+        expected = Series(list(range(10)), dtype='int64')
+        result = Series(Iter(), dtype='int64')
+        assert_series_equal(result, expected)
+
+    def test_constructor_sequence(self):
+        # GH 21987
         expected = Series(list(range(10)), dtype='int64')
         result = Series(range(10), dtype='int64')
         assert_series_equal(result, expected)
 
+    def test_constructor_single_str(self):
+        # GH 21987
+        expected = Series(['abc'])
+        result = Series('abc')
+        assert_series_equal(result, expected)
+
     def test_constructor_list_like(self):
 
         # make sure that we are coercing different
@@ -226,10 +255,13 @@ class TestSeriesConstructors(TestData):
         res = Series(cat)
         tm.assert_categorical_equal(res.values, cat)
 
+        # can cast to a new dtype
+        result = Series(pd.Categorical([1, 2, 3]),
+                        dtype='int64')
+        expected = pd.Series([1, 2, 3], dtype='int64')
+        tm.assert_series_equal(result, expected)
+
         # GH12574
-        pytest.raises(
-            ValueError, lambda: Series(pd.Categorical([1, 2, 3]),
-                                       dtype='int64'))
         cat = Series(pd.Categorical([1, 2, 3]), dtype='category')
         assert is_categorical_dtype(cat)
         assert is_categorical_dtype(cat.dtype)
@@ -423,6 +455,13 @@ class TestSeriesConstructors(TestData):
                            datetime(2001, 1, 3)], index=index, dtype='M8[ns]')
         assert_series_equal(result, expected)
 
+    def test_constructor_maskedarray_hardened(self):
+        # Check numpy masked arrays with hard masks -- from GH24574
+        data = ma.masked_all((3, ), dtype=float).harden_mask()
+        result = pd.Series(data)
+        expected = pd.Series([nan, nan, nan])
+        tm.assert_series_equal(result, expected)
+
     def test_series_ctor_plus_datetimeindex(self):
         rng = date_range('20090415', '20090519', freq='B')
         data = {k: 1 for k in rng}
@@ -445,7 +484,7 @@ class TestSeriesConstructors(TestData):
         # test that construction of a Series with an index of different length
         # raises an error
         msg = 'Length of passed values is 3, index implies 4'
-        with pytest.raises(ValueError, message=msg):
+        with pytest.raises(ValueError, match=msg):
             Series(input, index=np.arange(4))
 
     def test_constructor_numpy_scalar(self):
@@ -460,7 +499,9 @@ class TestSeriesConstructors(TestData):
         # GH 19342
         # construction with single-element container and index
         # should raise
-        pytest.raises(ValueError, Series, ['foo'], index=['a', 'b', 'c'])
+        msg = "Length of passed values is 1, index implies 3"
+        with pytest.raises(ValueError, match=msg):
+            Series(['foo'], index=['a', 'b', 'c'])
 
     def test_constructor_corner(self):
         df = tm.makeTimeDataFrame()
@@ -542,12 +583,30 @@ class TestSeriesConstructors(TestData):
         tm.assert_series_equal(Series(np.array([np.nan, pd.NaT])), exp)
 
     def test_constructor_cast(self):
-        pytest.raises(ValueError, Series, ['a', 'b', 'c'], dtype=float)
+        msg = "could not convert string to float"
+        with pytest.raises(ValueError, match=msg):
+            Series(["a", "b", "c"], dtype=float)
 
-    def test_constructor_dtype_nocast(self):
-        # 1572
+    def test_constructor_unsigned_dtype_overflow(self, uint_dtype):
+        # see gh-15832
+        msg = 'Trying to coerce negative values to unsigned integers'
+        with pytest.raises(OverflowError, match=msg):
+            Series([-1], dtype=uint_dtype)
+
+    def test_constructor_coerce_float_fail(self, any_int_dtype):
+        # see gh-15832
+        msg = "Trying to coerce float values to integers"
+        with pytest.raises(ValueError, match=msg):
+            Series([1, 2, 3.5], dtype=any_int_dtype)
+
+    def test_constructor_coerce_float_valid(self, float_dtype):
+        s = Series([1, 2, 3.5], dtype=float_dtype)
+        expected = Series([1, 2, 3.5]).astype(float_dtype)
+        assert_series_equal(s, expected)
+
+    def test_constructor_dtype_no_cast(self):
+        # see gh-1572
         s = Series([1, 2, 3])
-
         s2 = Series(s, dtype=np.int64)
 
         s2[1] = 5
@@ -622,10 +681,17 @@ class TestSeriesConstructors(TestData):
         assert s.dtype == 'M8[ns]'
 
         # GH3414 related
+        # msg = (r"cannot astype a datetimelike from \[datetime64\[ns\]\] to"
+        #        r" \[int32\]")
+        # with pytest.raises(TypeError, match=msg):
+        #     Series(Series(dates).astype('int') / 1000000, dtype='M8[ms]')
         pytest.raises(TypeError, lambda x: Series(
             Series(dates).astype('int') / 1000000, dtype='M8[ms]'))
-        pytest.raises(TypeError,
-                      lambda x: Series(dates, dtype='datetime64'))
+
+        msg = (r"The 'datetime64' dtype has no unit\. Please pass in"
+               r" 'datetime64\[ns\]' instead\.")
+        with pytest.raises(ValueError, match=msg):
+            Series(dates, dtype='datetime64')
 
         # invalid dates can be help as object
         result = Series([datetime(2, 1, 1)])
@@ -760,12 +826,12 @@ class TestSeriesConstructors(TestData):
         s = Series([pd.Timestamp('2013-01-01 13:00:00-0800', tz='US/Pacific'),
                     pd.Timestamp('2013-01-02 14:00:00-0800', tz='US/Pacific')])
         assert s.dtype == 'datetime64[ns, US/Pacific]'
-        assert lib.infer_dtype(s) == 'datetime64'
+        assert lib.infer_dtype(s, skipna=True) == 'datetime64'
 
         s = Series([pd.Timestamp('2013-01-01 13:00:00-0800', tz='US/Pacific'),
                     pd.Timestamp('2013-01-02 14:00:00-0800', tz='US/Eastern')])
         assert s.dtype == 'object'
-        assert lib.infer_dtype(s) == 'datetime'
+        assert lib.infer_dtype(s, skipna=True) == 'datetime'
 
         # with all NaT
         s = Series(pd.NaT, index=[0, 1], dtype='datetime64[ns, US/Eastern]')
@@ -820,17 +886,33 @@ class TestSeriesConstructors(TestData):
         result = Series(s.values, dtype=s.dtype)
         tm.assert_series_equal(result, s)
 
+    def test_constructor_infer_period(self):
+        data = [pd.Period('2000', 'D'), pd.Period('2001', 'D'), None]
+        result = pd.Series(data)
+        expected = pd.Series(period_array(data))
+        tm.assert_series_equal(result, expected)
+        assert result.dtype == 'Period[D]'
+
+        data = np.asarray(data, dtype=object)
+        tm.assert_series_equal(result, expected)
+        assert result.dtype == 'Period[D]'
+
+    def test_constructor_period_incompatible_frequency(self):
+        data = [pd.Period('2000', 'D'), pd.Period('2001', 'A')]
+        result = pd.Series(data)
+        assert result.dtype == object
+        assert result.tolist() == data
+
     def test_constructor_periodindex(self):
         # GH7932
         # converting a PeriodIndex when put in a Series
 
         pi = period_range('20130101', periods=5, freq='D')
         s = Series(pi)
+        assert s.dtype == 'Period[D]'
         expected = Series(pi.astype(object))
         assert_series_equal(s, expected)
 
-        assert s.dtype == 'object'
-
     def test_constructor_dict(self):
         d = {'a': 0., 'b': 1., 'c': 2.}
         result = Series(d, index=['b', 'c', 'd', 'a'])
@@ -914,11 +996,15 @@ class TestSeriesConstructors(TestData):
         tm.assert_series_equal(result, expected)
 
     def test_constructor_set(self):
-        values = set([1, 2, 3, 4, 5])
-        pytest.raises(TypeError, Series, values)
+        values = {1, 2, 3, 4, 5}
+        with pytest.raises(TypeError, match="'set' type is unordered"):
+            Series(values)
         values = frozenset(values)
-        pytest.raises(TypeError, Series, values)
+        with pytest.raises(TypeError, match="'frozenset' type is unordered"):
+            Series(values)
 
+    # https://github.com/pandas-dev/pandas/issues/22698
+    @pytest.mark.filterwarnings("ignore:elementwise comparison:FutureWarning")
     def test_fromDict(self):
         data = {'a': 0, 'b': 1, 'c': 2, 'd': 3}
 
@@ -937,27 +1023,27 @@ class TestSeriesConstructors(TestData):
         series = Series(data, dtype=float)
         assert series.dtype == np.float64
 
-    def test_fromValue(self):
+    def test_fromValue(self, datetime_series):
 
-        nans = Series(np.NaN, index=self.ts.index)
+        nans = Series(np.NaN, index=datetime_series.index)
         assert nans.dtype == np.float_
-        assert len(nans) == len(self.ts)
+        assert len(nans) == len(datetime_series)
 
-        strings = Series('foo', index=self.ts.index)
+        strings = Series('foo', index=datetime_series.index)
         assert strings.dtype == np.object_
-        assert len(strings) == len(self.ts)
+        assert len(strings) == len(datetime_series)
 
         d = datetime.now()
-        dates = Series(d, index=self.ts.index)
+        dates = Series(d, index=datetime_series.index)
         assert dates.dtype == 'M8[ns]'
-        assert len(dates) == len(self.ts)
+        assert len(dates) == len(datetime_series)
 
         # GH12336
         # Test construction of categorical series from value
-        categorical = Series(0, index=self.ts.index, dtype="category")
-        expected = Series(0, index=self.ts.index).astype("category")
+        categorical = Series(0, index=datetime_series.index, dtype="category")
+        expected = Series(0, index=datetime_series.index).astype("category")
         assert categorical.dtype == 'category'
-        assert len(categorical) == len(self.ts)
+        assert len(categorical) == len(datetime_series)
         tm.assert_series_equal(categorical, expected)
 
     def test_constructor_dtype_timedelta64(self):
@@ -1010,14 +1096,16 @@ class TestSeriesConstructors(TestData):
         td.astype('int64')
 
         # invalid casting
-        pytest.raises(TypeError, td.astype, 'int32')
+        msg = (r"cannot astype a timedelta from \[timedelta64\[ns\]\] to"
+               r" \[int32\]")
+        with pytest.raises(TypeError, match=msg):
+            td.astype('int32')
 
         # this is an invalid casting
-        def f():
+        msg = "Could not convert object to NumPy timedelta"
+        with pytest.raises(ValueError, match=msg):
             Series([timedelta(days=1), 'foo'], dtype='m8[ns]')
 
-        pytest.raises(Exception, f)
-
         # leave as object here
         td = Series([timedelta(days=i) for i in range(3)] + ['foo'])
         assert td.dtype == 'object'
@@ -1063,9 +1151,11 @@ class TestSeriesConstructors(TestData):
                 assert s.name == n
 
     def test_constructor_name_unhashable(self):
+        msg = r"Series\.name must be a hashable type"
         for n in [['name_list'], np.ones(2), {1: 2}]:
             for data in [['name_list'], np.ones(2), {1: 2}]:
-                pytest.raises(TypeError, Series, data, name=n)
+                with pytest.raises(TypeError, match=msg):
+                    Series(data, name=n)
 
     def test_auto_conversion(self):
         series = Series(list(date_range('1/1/2000', periods=10)))
@@ -1103,8 +1193,13 @@ class TestSeriesConstructors(TestData):
     def test_constructor_cant_cast_datetimelike(self, index):
 
         # floats are not ok
-        msg = "Cannot cast {} to ".format(type(index).__name__)
-        with tm.assert_raises_regex(TypeError, msg):
+        msg = "Cannot cast {}.*? to ".format(
+            # strip Index to convert PeriodIndex -> Period
+            # We don't care whether the error message says
+            # PeriodIndex or PeriodArray
+            type(index).__name__.rstrip("Index")
+        )
+        with pytest.raises(TypeError, match=msg):
             Series(index, dtype=float)
 
         # ints are ok
@@ -1134,32 +1229,26 @@ class TestSeriesConstructors(TestData):
         exp = Series(index).astype(object)
         tm.assert_series_equal(s, exp)
 
-    def test_constructor_generic_timestamp_deprecated(self):
-        # see gh-15524
+    @pytest.mark.parametrize("dtype", [
+        np.datetime64,
+        np.timedelta64,
+    ])
+    def test_constructor_generic_timestamp_no_frequency(self, dtype):
+        # see gh-15524, gh-15987
+        msg = "dtype has no unit. Please pass in"
 
-        with tm.assert_produces_warning(FutureWarning):
-            dtype = np.timedelta64
-            s = Series([], dtype=dtype)
+        with pytest.raises(ValueError, match=msg):
+            Series([], dtype=dtype)
 
-            assert s.empty
-            assert s.dtype == 'm8[ns]'
+    @pytest.mark.parametrize("dtype,msg", [
+        ("m8[ps]", "cannot convert timedeltalike"),
+        ("M8[ps]", "cannot convert datetimelike"),
+    ])
+    def test_constructor_generic_timestamp_bad_frequency(self, dtype, msg):
+        # see gh-15524, gh-15987
 
-        with tm.assert_produces_warning(FutureWarning):
-            dtype = np.datetime64
-            s = Series([], dtype=dtype)
-
-            assert s.empty
-            assert s.dtype == 'M8[ns]'
-
-        # These timestamps have the wrong frequencies,
-        # so an Exception should be raised now.
-        msg = "cannot convert timedeltalike"
-        with tm.assert_raises_regex(TypeError, msg):
-            Series([], dtype='m8[ps]')
-
-        msg = "cannot convert datetimelike"
-        with tm.assert_raises_regex(TypeError, msg):
-            Series([], dtype='M8[ps]')
+        with pytest.raises(TypeError, match=msg):
+            Series([], dtype=dtype)
 
     @pytest.mark.parametrize('dtype', [None, 'uint8', 'category'])
     def test_constructor_range_dtype(self, dtype):
@@ -1167,3 +1256,11 @@ class TestSeriesConstructors(TestData):
         expected = Series([0, 1, 2, 3, 4], dtype=dtype or 'int64')
         result = Series(range(5), dtype=dtype)
         tm.assert_series_equal(result, expected)
+
+    def test_constructor_tz_mixed_data(self):
+        # GH 13051
+        dt_list = [Timestamp('2016-05-01 02:03:37'),
+                   Timestamp('2016-04-30 19:03:37-0700', tz='US/Pacific')]
+        result = Series(dt_list)
+        expected = Series(dt_list, dtype=object)
+        tm.assert_series_equal(result, expected)
